# coding: latin-1
# Copyright (c) 2014 Dirk Baechle.
# www: https://bitbucket.org/dirkbaechle/pysketch
# mail: dl9obn AT darc.de
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

from Tkinter import *
import tkFileDialog
from PIL import Image, ImageDraw

import pysketch

# Offsets for drawing
rect_left = pysketch.pen_size / 2
rect_right = pysketch.pen_size - pysketch.pen_size/2
erase_left = rect_left * pysketch.erase_factor
erase_right = rect_right * pysketch.erase_factor

fgcolor = "black"
bgcolor = "white"
pilbg = (255, 255, 255)
if not pysketch.black_pen:
    fgcolor = "white"
    bgcolor = "black"
    pilbg = (0, 0, 0)

# Storing the last mouse position
lastx = None
lasty = None


class ResizingCanvas(Canvas):
    """ A subclass of Canvas for dealing with resizing and
        managing a PIL image in the background, for saving
        drawings to a file.
    """

    def __init__(self, parent, **kwargs):
        Canvas.__init__(self, parent, **kwargs)
        self.bind("<Configure>", self.on_resize)
        self.height = self.winfo_reqheight()
        self.width = self.winfo_reqwidth()

        # create a PIL canvas in memory and use in parallel
        self.pil_img = Image.new("RGB", (self.width, self.height), pilbg)
        self.pil_draw = ImageDraw.Draw(self.pil_img)

    def on_resize(self, event):
        """ Handle a resize by creating a new PIL image, and
            copying contents from the previous drawing over
            to it.
        """

        # create new PIL image
        tpil = Image.new("RGB", (event.width, event.height), pilbg)
        # Copy old image into new one
        copy_box = (0, 0, min(self.width, event.width), min(self.height, event.height))
        tpil.paste(self.pil_img, copy_box)
        self.pil_img = tpil
        self.pil_draw = ImageDraw.Draw(self.pil_img)

        self.width = event.width
        self.height = event.height
        # resize the canvas
        self.config(width=self.width, height=self.height)

    def draw_point(self, x, y, rectshape=True):
        """ Draw a single 'point' (=square/circle) at the given position.
        """

        if rectshape:
            self.create_rectangle((x - rect_left,
                                   y - rect_left,
                                   x + rect_right,
                                   y + rect_right),
                                  fill=fgcolor, outline=fgcolor)
            self.pil_draw.rectangle(((x - rect_left,
                                     y - rect_left),
                                     (x + rect_right,
                                      y + rect_right)),
                                    fill=fgcolor, outline=fgcolor)
        else:
            self.create_oval((x - rect_left,
                              y - rect_left,
                              x + rect_right,
                              y + rect_right),
                             fill=fgcolor, outline=fgcolor)
            self.pil_draw.ellipse(((x - rect_left,
                                    y - rect_left),
                                   (x + rect_right,
                                    y + rect_right)),
                                  fill=fgcolor, outline=fgcolor)

    def draw_at(self, x, y):
        """ Draw a single 'point' at, or a line to, the given position.
        """

        if not pysketch.draw_lines:
            self.draw_point(x, y)
        else:
            global lastx
            global lasty
            if lastx is not None:
                if lastx != x or lasty != y:
                    self.create_line(lastx, lasty, x, y,
                                     fill=fgcolor, width=pysketch.pen_size,
                                     capstyle=ROUND)
                    self.pil_draw.line((lastx, lasty, x, y),
                                       fill=fgcolor, width=pysketch.pen_size)
                    # Mimicking ROUND join/butt style, by drawing two additional circles
                    # at the end points...
                    self.pil_draw.ellipse(((x - rect_left,
                                            y - rect_left),
                                           (x + rect_right,
                                            y + rect_right)),
                                          fill=fgcolor, outline=fgcolor)
                    self.pil_draw.ellipse(((lastx - rect_left,
                                            lasty - rect_left),
                                           (lastx + rect_right,
                                            lasty + rect_right)),
                                          fill=fgcolor, outline=fgcolor)

            else:
                self.draw_point(x, y, False)

            # store the current position
            lastx = x
            lasty = y

    def erase_point(self, x, y, rectshape=True):
        """ Erase a single 'point' (=square/circle) at the given position.
        """

        if rectshape:
            self.create_rectangle((x - erase_left,
                                   y - erase_left,
                                   x + erase_right,
                                   y + erase_right),
                                  fill=bgcolor, outline=bgcolor)
            self.pil_draw.rectangle(((x - erase_left,
                                     y - erase_left),
                                    (x + erase_right,
                                     y + erase_right)),
                                    fill=bgcolor, outline=bgcolor)
        else:
            self.create_oval((x - erase_left,
                              y - erase_left,
                              x + erase_right,
                              y + erase_right),
                             fill=bgcolor, outline=bgcolor)
            self.pil_draw.ellipse(((x - erase_left,
                                    y - erase_left),
                                   (x + erase_right,
                                    y + erase_right)),
                                  fill=bgcolor, outline=bgcolor)

    def erase_at(self, x, y):
        """ Erase a single 'point' at, or a line to, the given position.
        """

        if not pysketch.draw_lines:
            self.erase_point(x, y)
        else:
            global lastx
            global lasty
            if lastx is not None:
                if lastx != x or lasty != y:
                    self.create_line(lastx, lasty, x, y,
                                     fill=bgcolor, width=pysketch.erase_factor*pysketch.pen_size,
                                     capstyle=ROUND)
                    self.pil_draw.line((lastx, lasty, x, y),
                                       fill=bgcolor,
                                       width=pysketch.erase_factor*pysketch.pen_size)
                    # Mimicking ROUND join/butt style, by drawing two additional circles
                    # at the end points...
                    self.pil_draw.ellipse(((x - erase_left,
                                            y - erase_left),
                                           (x + erase_right,
                                            y + erase_right)),
                                          fill=bgcolor, outline=bgcolor)
                    self.pil_draw.ellipse(((lastx - erase_left,
                                            lasty - erase_left),
                                           (lastx + erase_right,
                                            lasty + erase_right)),
                                          fill=bgcolor, outline=bgcolor)

            else:
                self.erase_point(x, y, False)

            # store the current position
            lastx = x
            lasty = y

    def release_button(self):
        """ Reset last mouse cursor position when the left or right
            mouse button was released again.
        """

        global lastx
        global lasty

        lastx = None
        lasty = None

    def save_image(self):
        """ Save the current drawing to a file.
        """

        fname = tkFileDialog.asksaveasfilename(parent=self,
                                               initialdir='.',
                                               filetypes=[('PNG files', '*.png')])

        if fname != "":
            if not fname.endswith('.png'):
                fname += ".png"

            # save image as PNG file
            self.pil_img.save(fname)

    def new_image(self):
        """ Start a new image by removing all canvas items, and
            clearing the PIL image with the background color.
        """

        self.delete("all")
        self.pil_draw.rectangle(((0, 0), (self.width, self.height)),
                                fill=bgcolor, outline=bgcolor)


class TkinterFrame:
    """ The main window frame.
    """

    def __init__(self, root):
        root.title('pysketch %s' % pysketch.version)

        self.widgets = {}
        self.canvas = ResizingCanvas(root,
                                     width=pysketch.canvas_width,
                                     height=pysketch.canvas_height,
                                     bg=bgcolor,
                                     highlightthickness=0)
        self.canvas.grid(column=0, columnspan=1, row=0, sticky='nesw')

        self.canvas.bind("<Button-1>", self.draw_brush)
        self.canvas.bind("<B1-Motion>", self.draw_brush)

        self.canvas.bind("<Button-3>", self.erase_brush)
        self.canvas.bind("<B3-Motion>", self.erase_brush)

        self.canvas.bind("<ButtonRelease-1>", self.release_button)
        self.canvas.bind("<ButtonRelease-3>", self.release_button)

        box = Frame(root)

        w = Button(box, text="Save", width=10, command=self.canvas.save_image)
        w.pack(side=LEFT, padx=5, pady=5, fill=X, expand=YES)
        w = Button(box, text="New", width=10, command=self.canvas.new_image)
        w.pack(side=LEFT, padx=5, pady=5, fill=X, expand=YES)

        box.grid(column=0, columnspan=1, row=1, sticky='ew')

        ## Resize behaviour(s)
        root.grid_rowconfigure(0, weight=1, minsize=pysketch.canvas_height)
        root.grid_rowconfigure(1, weight=0, minsize=30)
        root.grid_columnconfigure(0, weight=1, minsize=pysketch.canvas_width)

    def draw_brush(self, event):
        self.canvas.draw_at(event.x, event.y)

    def erase_brush(self, event):
        self.canvas.erase_at(event.x, event.y)

    def release_button(self, event):
        self.canvas.release_button()



# coding: latin-1
# Copyright (c) 2014 Dirk Baechle.
# www: https://bitbucket.org/dirkbaechle/pysketch
# mail: dl9obn AT darc.de
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import pygtk
pygtk.require('2.0')
import gtk
import gtk.gdk

import pysketch

#
# Roughly based on the simplescribble.py example from the pygtk examples
#

# Offsets for drawing
rect_left = pysketch.pen_size / 2
erase_left = rect_left * pysketch.erase_factor
erase_width = pysketch.pen_size * pysketch.erase_factor

# Backing pixmap for drawing area
pixmap = None
fg_gc = None
bg_gc = None

# Storing the last mouse position
lastx = None
lasty = None


def style_gc(w, black=True):
    """ Simple helper function for selecting the correct graphics
        context, based on the set polarity.
    """
    
    global fg_gc
    global bg_gc

    if fg_gc is None:
        # Get the styles only once, assuming that always
        # the same widget gets passed...
        style = w.get_style()
        if pysketch.black_pen:
            fg_gc = style.black_gc
            bg_gc = style.white_gc
        else:
            fg_gc = style.white_gc
            bg_gc = style.black_gc

        # Setup line styles
        if pysketch.draw_lines:
            fg_gc.line_width = pysketch.pen_size
            fg_gc.fill = gtk.gdk.SOLID
            fg_gc.function = gtk.gdk.COPY
            fg_gc.line_style = gtk.gdk.LINE_SOLID
            fg_gc.join_style = gtk.gdk.JOIN_ROUND
            fg_gc.cap_style = gtk.gdk.CAP_ROUND
            bg_gc.line_width = erase_width
            bg_gc.fill = gtk.gdk.SOLID
            bg_gc.function = gtk.gdk.COPY
            bg_gc.line_style = gtk.gdk.LINE_SOLID
            bg_gc.join_style = gtk.gdk.JOIN_ROUND
            bg_gc.cap_style = gtk.gdk.CAP_ROUND

    if black:
        return fg_gc
    else:
        return bg_gc


def configure_event(widget, event):
    """ Create a new backing pixmap of the appropriate size.
    """
    
    global pixmap

    x, y, width, height = widget.get_allocation()
    tpixmap = gtk.gdk.Pixmap(widget.window, width, height)
    tpixmap.draw_rectangle(style_gc(widget, black=False),
                           True, 0, 0, width, height)
    if pixmap:
        # Copy old contents to new bitmap
        colormap = pixmap.get_colormap()
        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, *pixmap.get_size())
        pixbuf = pixbuf.get_from_drawable(pixmap, colormap,
                                          0, 0, 0, 0,
                                          *pixmap.get_size())
        tpixmap.draw_pixbuf(widget.get_style().fg_gc[gtk.STATE_NORMAL], pixbuf,
                            0, 0, 0, 0)
    # Replace current bitmap
    pixmap = tpixmap

    return True


def expose_event(widget, event):
    """ Redraw the screen from the backing pixmap.
    """
    
    x, y, width, height = event.area
    widget.window.draw_drawable(widget.get_style().fg_gc[gtk.STATE_NORMAL],
                                pixmap, x, y, x, y, width, height)
    return False


def draw_point(widget, x, y, rectshape=True):
    """ Draw a point (=square/circle) on the screen.
    """
    
    rect = (x-rect_left, y-rect_left,
            pysketch.pen_size, pysketch.pen_size)
    if rectshape:
        pixmap.draw_rectangle(style_gc(widget, black=True), True,
                              rect[0], rect[1], rect[2], rect[3])
    else:
        pixmap.draw_arc(style_gc(widget, black=True), True,
                        rect[0], rect[1], rect[2], rect[3], 0, 64*360)
    widget.queue_draw_area(rect[0], rect[1], rect[2], rect[3])


def draw_brush(widget, x, y):
    """ Draw a line or point on the screen.
    """

    if not pysketch.draw_lines:
        draw_point(widget, x, y)
    else:
        global lastx
        global lasty
        if lastx is not None:
            if lastx != x or lasty != y:
                pixmap.draw_line(style_gc(widget, black=True),
                                 lastx, lasty, x, y)
                updx = min(lastx, x)
                updy = min(lasty, y)
                widget.queue_draw_area(updx-pysketch.pen_size,
                                       updy-pysketch.pen_size,
                                       abs(x-lastx)+2*pysketch.pen_size,
                                       abs(y-lasty)+2*pysketch.pen_size)
        else:
            draw_point(widget, x, y, False)

        # store the current position
        lastx = x
        lasty = y


def erase_point(widget, x, y, rectshape=True):
    """ Erase a point (=square/circle) on the screen.
    """
    
    rect = (x-erase_left, y-erase_left,
            erase_width, erase_width)
    if rectshape:
        pixmap.draw_rectangle(style_gc(widget, black=False), True,
                              rect[0], rect[1], rect[2], rect[3])
    else:
        pixmap.draw_arc(style_gc(widget, black=False), True,
                        rect[0], rect[1], rect[2], rect[3], 0, 64*360)

    widget.queue_draw_area(rect[0], rect[1], rect[2], rect[3])


def erase_brush(widget, x, y):
    """ Erase a line or point on the screen.
    """
    
    if not pysketch.draw_lines:
        erase_point(widget, x, y)
    else:
        global lastx
        global lasty
        if lastx is not None:
            if lastx != x or lasty != y:
                pixmap.draw_line(style_gc(widget, black=False),
                                 lastx, lasty, x, y)
                updx = min(lastx, x)
                updy = min(lasty, y)
                widget.queue_draw_area(updx-erase_width,
                                       updy-erase_width,
                                       abs(x-lastx)+2*erase_width,
                                       abs(y-lasty)+2*erase_width)
        else:
            erase_point(widget, x, y, False)

        # store the current position
        lastx = x
        lasty = y


def button_press_event(widget, event):
    """ React to a button press event.
    """
    
    if event.button == 1 and pixmap is not None:
        draw_brush(widget, int(event.x), int(event.y))
    elif event.button == 3 and pixmap is not None:
        erase_brush(widget, int(event.x), int(event.y))
    return True


def button_release_event(widget, event):
    """ React to a button release event.
    """
    
    global lastx
    global lasty

    lastx = None
    lasty = None

    return True


def motion_notify_event(widget, event):
    """ React to a motion event.
    """
    
    if event.is_hint:
        x, y, state = event.window.get_pointer()
    else:
        x = event.x
        y = event.y
        state = event.state

    if state & gtk.gdk.BUTTON1_MASK and pixmap is not None:
        draw_brush(widget, int(x), int(y))
    elif state & gtk.gdk.BUTTON3_MASK and pixmap is not None:
        erase_brush(widget, int(x), int(y))

    return True


class PygtkWindow(gtk.Window):
    """ The main window/app.
    """
    
    def __init__(self):
        super(PygtkWindow, self).__init__(gtk.WINDOW_TOPLEVEL)
        self.set_title("pysketch %s" % pysketch.version)

        tbox = gtk.Table(2, 1, False)
        self.add(tbox)
        tbox.show()

        self.connect("destroy", lambda w: gtk.main_quit())

        # Create the drawing area
        self.drawing_area = gtk.DrawingArea()
        self.drawing_area.set_size_request(pysketch.canvas_width,
                                           pysketch.canvas_height)

        tbox.attach(self.drawing_area, 0, 1, 0, 1,
                    gtk.FILL | gtk.EXPAND, gtk.FILL | gtk.EXPAND)

        self.drawing_area.show()

        # Signals used to handle backing pixmap
        self.drawing_area.connect("expose_event", expose_event)
        self.drawing_area.connect("configure_event", configure_event)

        # Event signals
        self.drawing_area.connect("motion_notify_event", motion_notify_event)
        self.drawing_area.connect("button_press_event", button_press_event)
        self.drawing_area.connect("button_release_event", button_release_event)

        self.drawing_area.set_events(gtk.gdk.EXPOSURE_MASK
                                     | gtk.gdk.LEAVE_NOTIFY_MASK
                                     | gtk.gdk.BUTTON_PRESS_MASK
                                     | gtk.gdk.BUTTON_RELEASE_MASK
                                     | gtk.gdk.POINTER_MOTION_MASK
                                     | gtk.gdk.POINTER_MOTION_HINT_MASK)

        # .. some buttons
        hbox = gtk.HBox(True, 3)
        sbutton = gtk.Button("Save")

        hbox.pack_start(sbutton, True, True, 0)
        sbutton.connect_object("clicked", lambda w: w.save_image(), self)
        sbutton.show()

        nbutton = gtk.Button("New")
        hbox.pack_start(nbutton, True, True, 0)
        nbutton.connect_object("clicked", lambda w: w.new_image(), self)
        nbutton.show()

        hbox.show()
        tbox.attach(hbox, 0, 1, 1, 2,
                    gtk.FILL | gtk.EXPAND, gtk.FILL)

    def save_image(self):
        """ Save the current sketch as PNG image file.
        """

        #
        # Get pix buffer first...
        #
        drawable = self.drawing_area.window
        colormap = drawable.get_colormap()
        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, *drawable.get_size())
        pixbuf = pixbuf.get_from_drawable(drawable, colormap,
                                          0, 0, 0, 0,
                                          *drawable.get_size())

        #
        # ...then start the SAVE dialog
        #
        dialog_buttons = (gtk.STOCK_CANCEL,
                          gtk.RESPONSE_CANCEL,
                          gtk.STOCK_SAVE,
                          gtk.RESPONSE_OK)

        file_dialog = gtk.FileChooserDialog(title="Select image file",
                                            action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                            buttons=dialog_buttons)
        # Create and add the file filter
        filter = gtk.FileFilter()
        filter.set_name("PNG file")
        filter.add_pattern("*.png")
        file_dialog.add_filter(filter)

        # Init the return value
        result = ""
        if file_dialog.run() == gtk.RESPONSE_OK:
            result = file_dialog.get_filename()
            if not result.endswith('.png'):
                result += '.png'

            pixbuf.save(result, 'png')

        file_dialog.destroy()

    def new_image(self):
        """ Start a new sketch by clearing the canvas.
        """
        
        if pixmap:
            sizes = pixmap.get_size()
            pixmap.draw_rectangle(style_gc(self.drawing_area, black=False),
                                  True, 0, 0, *sizes)
            self.drawing_area.queue_draw_area(0, 0, *sizes)



# coding: latin-1
# Copyright (c) 2014 Dirk Baechle.
# www: https://bitbucket.org/dirkbaechle/pysketch
# mail: dl9obn AT darc.de
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import math
import cairo
from gi.repository import Gtk, Gdk

import pysketch

#
# Roughly based on the drawingarea.py example from the pygobject demo
#

# Offsets for drawing
rect_left = pysketch.pen_size / 2
erase_left = rect_left * pysketch.erase_factor
erase_width = pysketch.pen_size * pysketch.erase_factor

fgcolor = (0, 0, 0)
bgcolor = (1.0, 1.0, 1.0)
if not pysketch.black_pen:
    fgcolor = (1.0, 1.0, 1.0)
    bgcolor = (0, 0, 0)


class PygobjectApp:

    def __init__(self):
        self.window = Gtk.Window()
        self.window.set_title('pysketch %s' % pysketch.version)
        self.window.set_default_size(pysketch.canvas_width, pysketch.canvas_height)
        self.window.connect('destroy', lambda x: Gtk.main_quit())
        self.width = pysketch.canvas_width
        self.height = pysketch.canvas_height

        # Storing the last mouse position
        self.lastx = None
        self.lasty = None

        tbox = Gtk.Table(2, 1, False)
        self.window.add(tbox)

        # Create the drawing area
        self.surface = None
        self.drawing_area = Gtk.DrawingArea()
        self.drawing_area.set_size_request(pysketch.canvas_width,
                                           pysketch.canvas_height)

        tbox.attach(self.drawing_area, 0, 1, 0, 1,
                    Gtk.AttachOptions.FILL | Gtk.AttachOptions.EXPAND,
                    Gtk.AttachOptions.FILL | Gtk.AttachOptions.EXPAND)

        # Signals used to handle backing pixmap
        self.drawing_area.connect("draw", self.draw_event)
        self.drawing_area.connect("configure-event", self.configure_event)

        # Event signals
        self.drawing_area.connect("motion-notify-event", self.motion_notify_event)
        self.drawing_area.connect("button-press-event", self.button_press_event)
        self.drawing_area.connect("button-release-event", self.button_release_event)

        self.drawing_area.set_events(self.drawing_area.get_events()
                                     | Gdk.EventMask.LEAVE_NOTIFY_MASK
                                     | Gdk.EventMask.BUTTON_PRESS_MASK
                                     | Gdk.EventMask.BUTTON_RELEASE_MASK
                                     | Gdk.EventMask.POINTER_MOTION_MASK
                                     | Gdk.EventMask.POINTER_MOTION_HINT_MASK)

        # .. some buttons
        hbox = Gtk.HBox(True, 3)
        sbutton = Gtk.Button("Save")

        hbox.pack_start(sbutton, True, True, 0)
        sbutton.connect("clicked", self.save_image)

        nbutton = Gtk.Button("New")
        hbox.pack_start(nbutton, True, True, 0)
        nbutton.connect("clicked", self.new_image)

        tbox.attach(hbox, 0, 1, 1, 2,
                    Gtk.AttachOptions.FILL | Gtk.AttachOptions.EXPAND,
                    Gtk.AttachOptions.FILL)

        self.window.show_all()

    def draw_event(self, da, cairo_ctx):
        cairo_ctx.set_source_surface(self.surface, 0, 0)
        cairo_ctx.paint()

        return False

    def cairo_line(self, cr,
                   lastx, lasty, x, y,
                   color, width):
        """ Draws a line with the given cairo context.
        """
        
        cr.set_line_cap(cairo.LINE_CAP_ROUND)
        cr.set_line_join(cairo.LINE_JOIN_BEVEL)
        cr.set_line_width(width)
        cr.set_source_rgb(*color)
        cr.move_to(lastx, lasty)
        cr.line_to(x, y)
        cr.stroke()

    def cairo_circle(self, cr, x, y, color, width):
        """ Draws a filled circle with the given cairo context.
        """
        
        cr.set_line_width(0.1)
        cr.set_source_rgb(*color)
        cr.arc(x, y, width/2.0, 0, 2 * math.pi)
        cr.stroke_preserve()
        cr.fill()

    def draw_point(self, widget, x, y, rectshape=True):
        """ Draw a point (=square/circle) on the screen.
        """
        
        update_rect = Gdk.Rectangle()
        update_rect.x = x - rect_left
        update_rect.y = y - rect_left
        update_rect.width = pysketch.pen_size
        update_rect.height = pysketch.pen_size

        # paint to the surface where we store our state
        cairo_ctx = cairo.Context(self.surface)
        if rectshape:
            cairo_ctx.set_source_rgb(*fgcolor)
            Gdk.cairo_rectangle(cairo_ctx, update_rect)
            cairo_ctx.fill()
        else:
            self.cairo_circle(cairo_ctx, x, y, fgcolor, pysketch.pen_size)

        widget.get_window().invalidate_rect(update_rect, False)

    def draw_brush(self, widget, x, y, rectshape=True):
        """ Draw a line or point on the screen.
        """
        
        if not pysketch.draw_lines:
            self.draw_point(widget, x, y)
        else:
            if self.lastx is not None:
                if self.lastx != x or self.lasty != y:
                    cairo_ctx = cairo.Context(self.surface)
                    self.cairo_line(cairo_ctx,
                                    self.lastx, self.lasty, x, y,
                                    fgcolor, pysketch.pen_size)
                    updx = min(self.lastx, x)
                    updy = min(self.lasty, y)
                    update_rect = Gdk.Rectangle()
                    update_rect.x = updx-pysketch.pen_size
                    update_rect.y = updy-pysketch.pen_size
                    update_rect.width = abs(x-self.lastx)+2*pysketch.pen_size
                    update_rect.height = abs(y-self.lasty)+2*pysketch.pen_size
                    widget.get_window().invalidate_rect(update_rect, False)
            else:
                self.draw_point(widget, x, y, False)

            # store the current position
            self.lastx = x
            self.lasty = y

    def erase_point(self, widget, x, y, rectshape=True):
        """ Erase a point (=square/circle) on the screen.
        """
        
        update_rect = Gdk.Rectangle()
        update_rect.x = x - erase_left
        update_rect.y = y - erase_left
        update_rect.width = erase_width
        update_rect.height = erase_width

        # paint to the surface where we store our state
        cairo_ctx = cairo.Context(self.surface)
        if rectshape:
            cairo_ctx.set_source_rgb(*bgcolor)
            Gdk.cairo_rectangle(cairo_ctx, update_rect)
            cairo_ctx.fill()
        else:
            self.cairo_circle(cairo_ctx, x, y, bgcolor, erase_width)

        widget.get_window().invalidate_rect(update_rect, False)

    def erase_brush(self, widget, x, y, rectshape=True):
        """ Erase a line or point on the screen.
        """
        
        if not pysketch.draw_lines:
            self.erase_point(widget, x, y)
        else:
            if self.lastx is not None:
                if self.lastx != x or self.lasty != y:
                    cairo_ctx = cairo.Context(self.surface)
                    self.cairo_line(cairo_ctx,
                                    self.lastx, self.lasty, x, y,
                                    bgcolor, erase_width)
                    updx = min(self.lastx, x)
                    updy = min(self.lasty, y)
                    update_rect = Gdk.Rectangle()
                    update_rect.x = updx-erase_width
                    update_rect.y = updy-erase_width
                    update_rect.width = abs(x-self.lastx)+2*erase_width
                    update_rect.height = abs(y-self.lasty)+2*erase_width
                    widget.get_window().invalidate_rect(update_rect, False)
            else:
                self.erase_point(widget, x, y, False)

            # store the current position
            self.lastx = x
            self.lasty = y

    def configure_event(self, da, event):
        """ Windows resize -> create a new Surface of the appropriate size.
        """

        allocation = da.get_allocation()
        # Create new surface
        tmp_surface = da.get_window().create_similar_surface(cairo.CONTENT_COLOR,
                                                             allocation.width,
                                                             allocation.height)
        # Clear surface background
        cairo_ctx = cairo.Context(tmp_surface)
        cairo_ctx.set_source_rgb(*bgcolor)
        cairo_ctx.paint()
        if self.surface is not None:
            # Copy over old image
            cairo_ctx.set_source_surface(self.surface, 0, 0)
            cairo_ctx.paint()

        # Keep reference to surface
        self.width = allocation.width
        self.height = allocation.height
        self.surface = tmp_surface

        return True

    def motion_notify_event(self, da, event):
        """ React to a motion event.
        """

        if self.surface is None:  # paranoia check, in case we haven't gotten a configure event
            return False

        # This call is very important; it requests the next motion event.
        # If you don't call gdk_window_get_pointer() you'll only get
        # a single motion event. The reason is that we specified
        # GDK_POINTER_MOTION_HINT_MASK to gtk_widget_set_events().
        # If we hadn't specified that, we could just use event->x, event->y
        # as the pointer location. But we'd also get deluged in events.
        # By requesting the next event as we handle the current one,
        # we avoid getting a huge number of events faster than we
        # can cope.

        (window, x, y, state) = event.window.get_pointer()

        if state & Gdk.ModifierType.BUTTON1_MASK:
            self.draw_brush(da, x, y)
        elif state & Gdk.ModifierType.BUTTON3_MASK:
            self.erase_brush(da, x, y)

        return True

    def button_press_event(self, da, event):
        """ React to a button press event.
        """

        if self.surface is None:  # paranoia check, in case we haven't gotten a configure event
            return False

        if event.button == 1:
            self.draw_brush(da, event.x, event.y)
        elif event.button == 3:
            self.erase_brush(da, event.x, event.y)

        return True

    def button_release_event(self, da, event):
        """ React to a button release event.
        """
        
        self.lastx = None
        self.lasty = None

        return True

    def save_image(self, widget):
        """ Save the current sketch as PNG image file.
        """

        dialog = Gtk.FileChooserDialog("Please choose a file", self.window,
                                       Gtk.FileChooserAction.SAVE,
                                       (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                                        Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
        dialog.set_do_overwrite_confirmation(True)

        filter_text = Gtk.FileFilter()
        filter_text.set_name("PNG files")
        filter_text.add_mime_type("image/png")
        filter_text.add_pattern("*.png")
        dialog.add_filter(filter_text)

        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            fname = dialog.get_filename()
            if not fname.endswith('.png'):
                fname += '.png'
            if self.surface is not None:
                self.surface.write_to_png(fname)

        dialog.destroy()

    def new_image(self, widget):
        """ Start a new sketch by clearing the canvas.
        """

        update_rect = Gdk.Rectangle()
        update_rect.x = 0
        update_rect.y = 0
        update_rect.width = self.width
        update_rect.height = self.height

        # paint to the surface where we store our state
        cairo_ctx = cairo.Context(self.surface)
        cairo_ctx.set_source_rgb(*bgcolor)
        Gdk.cairo_rectangle(cairo_ctx, update_rect)
        cairo_ctx.fill()

        widget.get_window().invalidate_rect(update_rect, False)



# coding: latin-1
# Copyright (c) 2014 Dirk Baechle.
# www: https://bitbucket.org/dirkbaechle/pysketch
# mail: dl9obn AT darc.de
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#


def name():
    return "pyqt4"


def exists():
    try:
        import sip
        sip.setapi('QVariant', 2)

        from PyQt4 import QtCore, QtGui
    except:
        return False

    return True


def run_main():
    import pysketch.pyqt4.gui
    from PyQt4 import QtCore, QtGui

    app = QtGui.QApplication([])
    window = pysketch.pyqt4.gui.PyQt4Window()
    window.show()
    app.exec_()



# coding: latin-1
# Copyright (c) 2014 Dirk Baechle.
# www: https://bitbucket.org/dirkbaechle/pysketch
# mail: dl9obn AT darc.de
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import sip
sip.setapi('QVariant', 2)

from PyQt4 import QtCore, QtGui

import pysketch

#
# Roughly based on scribble.py from the PyQt4 examples
#

# Offsets for drawing
rect_left = pysketch.pen_size / 2
erase_left = rect_left * pysketch.erase_factor
erase_width = pysketch.pen_size * pysketch.erase_factor

fgcolor = QtCore.Qt.black
bgcolor = QtCore.Qt.white
if not pysketch.black_pen:
    fgcolor = QtCore.Qt.white
    bgcolor = QtCore.Qt.black


class ScribbleArea(QtGui.QWidget):
    def __init__(self, parent=None):
        super(ScribbleArea, self).__init__(parent)

        self.setAttribute(QtCore.Qt.WA_StaticContents)

        self.scribbling = False
        self.image = QtGui.QImage()
        self.fgbrush = QtGui.QBrush(fgcolor, QtCore.Qt.SolidPattern)
        self.bgbrush = QtGui.QBrush(bgcolor, QtCore.Qt.SolidPattern)
        self.lastPoint = QtCore.QPoint()

    def saveImage(self, fileName):
        """ Save the current sketch as PNG image
            under the given filename.
        """

        if self.image.save(fileName, 'png'):
            return True
        else:
            return False

    def newImage(self):
        """ Start a new sketch by clearing the canvas.
        """

        self.image.fill(bgcolor)
        self.update()

    def mousePressEvent(self, event):
        """ React to a button press event.
        """

        if event.button() == QtCore.Qt.LeftButton:
            self.lastPoint = event.pos()
            self.draw_brush(event.pos())
            self.scribbling = True
        elif event.button() == QtCore.Qt.RightButton:
            self.lastPoint = event.pos()
            self.erase_brush(event.pos())
            self.scribbling = True

    def mouseMoveEvent(self, event):
        """ React to a mouse move event.
        """

        if (event.buttons() & QtCore.Qt.LeftButton) and self.scribbling:
            self.draw_brush(event.pos())
        elif (event.buttons() & QtCore.Qt.RightButton) and self.scribbling:
            self.erase_brush(event.pos())

    def mouseReleaseEvent(self, event):
        """ React to a button release event.
        """

        if (((event.button() == QtCore.Qt.RightButton) or
             (event.button() == QtCore.Qt.LeftButton)) and self.scribbling):
            self.scribbling = False

    def paintEvent(self, event):
        """ Redraw the screen from the backing pixmap.
        """

        painter = QtGui.QPainter(self)
        painter.drawImage(QtCore.QPoint(0, 0), self.image)

    def resizeEvent(self, event):
        if self.width() != self.image.width() or self.height() != self.image.height():
            self.resizeImage(self.image, QtCore.QSize(self.width(), self.height()))
            self.update()

        super(ScribbleArea, self).resizeEvent(event)

    def draw_point(self, endPoint, rectshape=True):
        """ Draw a point (=square/circle) on the screen.
        """

        painter = QtGui.QPainter(self.image)
        painter.setPen(QtGui.QPen(fgcolor, 0.1,
                                  QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
        painter.setBrush(self.fgbrush)
        update_rect = QtCore.QRect(endPoint.x()-rect_left,
                                   endPoint.y()-rect_left,
                                   pysketch.pen_size, pysketch.pen_size)
        if rectshape:
            painter.drawRect(update_rect)
        else:
            painter.drawEllipse(update_rect)
        self.update(update_rect)

    def draw_brush(self, endPoint):
        """ Draw a line or point on the screen.
        """

        if not pysketch.draw_lines:
            self.draw_point(endPoint)
        else:
            if self.scribbling:
                if endPoint != self.lastPoint:

                    painter = QtGui.QPainter(self.image)
                    painter.setPen(QtGui.QPen(fgcolor, pysketch.pen_size,
                                              QtCore.Qt.SolidLine,
                                              QtCore.Qt.RoundCap,
                                              QtCore.Qt.RoundJoin))
                    painter.drawLine(self.lastPoint, endPoint)

                    rad = pysketch.pen_size / 2 + 2
                    self.update(QtCore.QRect(self.lastPoint, endPoint).normalized().adjusted(-rad, -rad, +rad, +rad))
            else:
                self.draw_point(endPoint, False)

            # store the current position
            self.lastPoint = QtCore.QPoint(endPoint)

    def erase_point(self, endPoint, rectshape=True):
        """ Erase a point (=square/circle) on the screen.
        """

        painter = QtGui.QPainter(self.image)
        painter.setPen(QtGui.QPen(bgcolor, 0.1,
                                  QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
        painter.setBrush(self.bgbrush)
        update_rect = QtCore.QRect(endPoint.x()-erase_left,
                                   endPoint.y()-erase_left,
                                   erase_width, erase_width)
        if rectshape:
            painter.drawRect(update_rect)
        else:
            painter.drawEllipse(update_rect)
        self.update(update_rect)

    def erase_brush(self, endPoint):
        """ Erase a line or point on the screen.
        """

        if not pysketch.draw_lines:
            self.erase_point(endPoint)
        else:
            if self.scribbling:
                if endPoint != self.lastPoint:

                    painter = QtGui.QPainter(self.image)
                    painter.setPen(QtGui.QPen(bgcolor, erase_width,
                                              QtCore.Qt.SolidLine,
                                              QtCore.Qt.RoundCap,
                                              QtCore.Qt.RoundJoin))
                    painter.drawLine(self.lastPoint, endPoint)

                    rad = erase_width / 2 + 2
                    self.update(QtCore.QRect(self.lastPoint, endPoint).normalized().adjusted(-rad, -rad, +rad, +rad))
            else:
                self.erase_point(endPoint, False)

            # store the current position
            self.lastPoint = QtCore.QPoint(endPoint)

    def resizeImage(self, image, newSize):
        """ Create a new backing pixmap of the appropriate size.
        """

        newImage = QtGui.QImage(newSize, QtGui.QImage.Format_RGB32)
        newImage.fill(bgcolor)
        painter = QtGui.QPainter(newImage)
        painter.drawImage(QtCore.QPoint(0, 0), image)
        self.image = newImage


class PyQt4Window(QtGui.QMainWindow):
    def __init__(self):
        super(PyQt4Window, self).__init__()

        self.scribbleArea = ScribbleArea()

        self.panel = QtGui.QWidget()
        self.vbox = QtGui.QVBoxLayout()
        self.hbox = QtGui.QHBoxLayout()
        self.save = QtGui.QPushButton("Save")
        self.new = QtGui.QPushButton("New")
        self.hbox.addWidget(self.save)
        self.hbox.addWidget(self.new)
        self.vbox.addWidget(self.scribbleArea)
        self.vbox.addLayout(self.hbox)
        self.panel.setLayout(self.vbox)

        self.setCentralWidget(self.panel)

        self.setWindowTitle("pysketch %s" % pysketch.version)
        self.scribbleArea.setMinimumSize(pysketch.canvas_width, pysketch.canvas_height)

        self.connect(self.save, QtCore.SIGNAL('clicked()'), self.saveImage)
        self.connect(self.new, QtCore.SIGNAL('clicked()'), self.scribbleArea.newImage)

    def saveImage(self):
        """ Get a filename and save the current sketch
            as PNG image file.
        """

        initialPath = QtCore.QDir.currentPath() + '/untitled.png'

        fileName = QtGui.QFileDialog.getSaveFileName(self, "Save As",
                                                     initialPath,
                                                     "PNG Files (*.png);;All Files (*)")
        if fileName:
            return self.scribbleArea.saveImage(fileName)

        return False



#!/usr/bin/env python

# coding: latin-1
# Copyright (c) 2014 Dirk Baechle.
# www: https://bitbucket.org/dirkbaechle/pysketch
# mail: dl9obn AT darc.de
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

from setuptools import setup

setup(name='pysketch',
      version='1.2',
      description='Draw simple notes and scribbles on a canvas',
      author='Dirk Baechle',
      author_email='dl9obn@darc.de',
      url='https://bitbucket.org/dirkbaechle/pysketch',
      packages=['pysketch',
                'pysketch/pygobject',
                'pysketch/pygtk',
                'pysketch/pyqt4',
                'pysketch/tkinter'],
      package_dir={'': 'src'},
      scripts=['pysketch'],
      license='MIT',
      classifiers=['Development Status :: 3 - Alpha',
                   'Environment :: X11 Applications',
                   'Intended Audience :: End Users/Desktop',
                   'Topic :: Utilities',
                   'License :: OSI Approved :: MIT License',
                   'Programming Language :: Python :: 2'
                  ],
      keywords='desktop note scribble',
      )



# coding: latin-1
# Copyright (c) 2014 Dirk Baechle.
# www: https://bitbucket.org/dirkbaechle/pysketch
# mail: dl9obn AT darc.de
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#
# Default values
#
canvas_width = 300
canvas_height = 300
pen_size = 3
erase_factor = 8
black_pen = True
draw_lines = True
version = '1.2'


def banner():
    """ Return a small banner message.
    """
    
    return "pysketch %s, https://bitbucket.org/dirkbaechle/pysketch" % version

#
# In this main package "pysketch", you can find the single subfolders for
# each supported GUI binding. The single GUI packages have to support
# the following interface:
#
#    package.exists()   : returns 'True' if all required modules can be imported,
#                         signalling that this binding is available
#    package.name()     : returns the unique name, that is used to reference this
#                         module throughout "pysketch"
#    package.run_main() : starts the main application for this binding type and
#                         runs it
#
# Each GUI module should try to mimic the following basic behaviour, as closely
# as possible:
#
# - The main window opens, such that the drawing area has the specified width/height.
# - There is no menu bar.
# - The window can get resized larger, but not smaller than the size specified by
#   the user or the default settings, respectively.
# - On a resize, the buttons in the bottom row always keep their minimum height.
# - The buttons fill/expand in the horizontal direction, a small padding between the single
#   buttons is allowed (e.g. 3px).
#
